/*
 * Copyright (C) 2009 Ericsson AB
 *
 * Authors: Jonas Sjöquist <jonas.sjoquist@ericsson.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <glib/gi18n.h>
#include "gps_control_log_window.h"

#define SAVE_FOLDER "/tmp"
#define SAVE_FILE "nmea_log.txt"

extern void log_window_clear_cb (GtkButton * button, gpointer udata)
{
	GtkTextBuffer *old_buffer, *new_buffer;

	old_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (nmea_data_text_view));
	new_buffer = gtk_text_buffer_new (NULL);

	if (!new_buffer) {
		g_error ("Unable to allocate new buffer!");
		return;
	}

	gtk_text_view_set_buffer (GTK_TEXT_VIEW (nmea_data_text_view), new_buffer);
	g_object_unref (old_buffer);
}

static void save_to_file (const char *filename)
{
	FILE *f;
	GtkTextBuffer *buffer;
	GtkTextIter start, end;
	gchar *temp_buffer;
	ssize_t written;

	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (nmea_data_text_view));
	if (!buffer) {
		g_warning ("Unable to get buffer!");
		return;
	}

	f = fopen (filename, "a");

	if (!f) {
		g_warning ("Unable to open file: %s", filename);
		return;
	}

	gtk_text_buffer_get_bounds (buffer, &start, &end);

	temp_buffer = gtk_text_buffer_get_text (buffer, &start, &end, TRUE);

	written = fwrite (temp_buffer, strlen (temp_buffer), 1, f);

	if (written < 0)
		g_warning ("Unable to write to file!");

	fclose (f);
}

extern void log_window_save_cb (GtkButton * button, gpointer udata)
{
	GtkWidget *dialog;
	char *filename;

	dialog = gtk_file_chooser_dialog_new (_("Save Log File"),
										  GTK_WINDOW (log_window),
										  GTK_FILE_CHOOSER_ACTION_SAVE,
										  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
										  GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
										  NULL);
	gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog),
													TRUE);
	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), SAVE_FOLDER);
	gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog),
									   SAVE_FILE);

	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
		save_to_file (filename);
		g_free (filename);
	}
	gtk_widget_destroy (dialog);
}

extern void log_window_append (char *buf, int len)
{
	GtkTextBuffer *buffer;
	GtkTextIter iter;

	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (nmea_data_text_view));
	gtk_text_buffer_get_end_iter (buffer, &iter);
	gtk_text_buffer_insert (buffer, &iter, buf, len);
	gtk_text_view_scroll_mark_onscreen (GTK_TEXT_VIEW (nmea_data_text_view),
										gtk_text_buffer_get_insert (buffer));
}

extern void show_log_window ()
{
	gtk_widget_show (log_window);
}

extern void hide_log_window ()
{
	gtk_widget_hide (log_window);
}
