/*
 * Copyright (C) 2009 Ericsson AB
 *
 * Authors: Torgny Johansson <torgny.johansson@ericsson.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#include <gtk/gtk.h>
#include <dbus/dbus.h>
#include <dbus/dbus-glib-lowlevel.h>
#include <dbus/dbus-glib.h>
#include <string.h>
#include <glib/gi18n.h>

#include "gps_control_supl_certificate.h"

#define SUPL_INST_CERT_NOT_STARTED 0
#define SUPL_INST_CERT_RUNNING 1
#define SUPL_INST_CERT_FAILED 2
#define SUPL_INST_CERT_SUCCESS 3

GtkWidget *install_certificate_window;
GtkWidget *cert_file_entry;
GtkWidget *cert_select_button;
GtkWidget *cert_install_button;
GtkWidget *cert_cancel_button;

char *filename;

DBUS_Connection *g_conn;

static void supl_cert_show_result_dialog (int result)
{
	GtkWidget *dialog;
	
	if (result == SUPL_INST_CERT_FAILED) {
		dialog =
			gtk_message_dialog_new (GTK_WINDOW (install_certificate_window),
									GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR,
									GTK_BUTTONS_OK,
									_("Unable to install SUPL certificate"));
		gtk_window_set_title (GTK_WINDOW (dialog), _("Error"));
		gtk_dialog_run (GTK_DIALOG (dialog));
		gtk_widget_destroy (dialog);
		gtk_widget_hide (install_certificate_window);
	} else if (result == SUPL_INST_CERT_SUCCESS) {
		dialog =
			gtk_message_dialog_new (GTK_WINDOW (install_certificate_window),
									GTK_DIALOG_MODAL, GTK_MESSAGE_INFO,
									GTK_BUTTONS_OK,
									_
									("SUPL certificate installed successfully"));
		gtk_window_set_title (GTK_WINDOW (dialog), _("Success"));
		gtk_dialog_run (GTK_DIALOG (dialog));
		gtk_widget_destroy (dialog);
		gtk_widget_hide (install_certificate_window);
	} else {
		g_debug ("Unknown result %d\n", result);
		gtk_widget_hide (install_certificate_window);
	}
}

extern void supl_cert_show_window ()
{
	gtk_widget_show (install_certificate_window);
}

extern void supl_cert_init_window (GtkBuilder * builder, DBUS_Connection * conn)
{
	install_certificate_window =
		GTK_WIDGET (gtk_builder_get_object
					(builder, "install_certificate_window"));
	cert_file_entry =
		GTK_WIDGET (gtk_builder_get_object (builder, "cert_file_entry"));
	cert_select_button =
		GTK_WIDGET (gtk_builder_get_object (builder, "cert_select_button"));
	cert_install_button =
		GTK_WIDGET (gtk_builder_get_object (builder, "cert_install_button"));
	cert_cancel_button =
		GTK_WIDGET (gtk_builder_get_object (builder, "cert_cancel_button"));

	g_conn = conn;
}

extern void on_cert_install_button_clicked (GtkButton * button,
											gpointer user_data)
{
	int status;
	
	dbus_setter_str (g_conn, "SuplInstallCertificate", filename);
	g_debug ("sent %s", filename);

	status = dbus_getter_int (g_conn, "GetSuplInstallCertificateStatus");
	while (status == SUPL_INST_CERT_RUNNING)
		status = dbus_getter_int (g_conn, "GetSuplInstallCertificateStatus");

	supl_cert_show_result_dialog (status);
}

static void supl_cert_show_open_dialog ()
{
	GtkWidget *dialog;

	dialog = gtk_file_chooser_dialog_new (_("Open certificate"),
										  GTK_WINDOW
										  (install_certificate_window),
										  GTK_FILE_CHOOSER_ACTION_OPEN,
										  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
										  GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
										  NULL);
	
	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog),
										 g_get_home_dir ());
	
	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
		gtk_entry_set_text (GTK_ENTRY (cert_file_entry), filename);
		g_debug ("Filename: %s", filename);
	}
	
	gtk_widget_destroy (dialog);
}

extern void on_cert_select_button_clicked (GtkButton * button,
										   gpointer user_data)
{
	supl_cert_show_open_dialog ();
}

extern void on_cert_file_entry_changed (GtkEntry * entry, gpointer user_data)
{
	if (strlen (gtk_entry_get_text (entry)) > 0) {
		gtk_widget_set_sensitive (cert_install_button, TRUE);
		filename = (char *)gtk_entry_get_text (entry);
	} else {
		gtk_widget_set_sensitive (cert_install_button, FALSE);
		filename = "";
	}
}
